<div class="subscribe-info form-group">
  <ul class="subscribe-list">
    <li data-toggle="tooltip" title="Detailed 6hr, 24hr, 7day, 30day, 1yr graphs of upload and download bandwidth for each server">
      <span class="glyphicon glyphicon-stats"></span>
      <span class="subscribe-feature">Server bandwidth graphs</span>
    </li>
    <li data-toggle="tooltip" title="Search for users in each organization">
      <span class="glyphicon glyphicon-search"></span>
      <span class="subscribe-feature">User searching</span>
    </li>
    <li data-toggle="tooltip" title="Bulk add new users with a list of usernames and email addresses">
      <span class="glyphicon glyphicon-th-list"></span>
      <span class="subscribe-feature">Bulk add users</span>
    </li>
    <li data-toggle="tooltip" title="Show static IP address, bandwidth usage and connection time of a user for each server">
      <span class="glyphicon glyphicon-list-alt"></span>
      <span class="subscribe-feature">Detailed user information</span>
    </li>
    <li data-toggle="tooltip" title="All users are assigned a static IP address on the servers network">
      <span class="glyphicon glyphicon-globe"></span>
      <span class="subscribe-feature">Static user IP addresses</span>
    </li>
    <li data-toggle="tooltip" title="Automatically email temporary links using Postmark to a user to access their key and two-step authentication code without logging in to the admin interface">
      <span class="glyphicon glyphicon-link"></span>
      <span class="subscribe-feature">Email users temporary key links</span>
    </li>
  </ul>
</div>
<div class="subscribe-activate-form form-group">
  <label>Enter License Key</label><br>
  <textarea rows="6" cols="35" spellcheck="false"></textarea>
</div>
<div class="subscribe-buttons">
  <button type="button" class="subscribe-checkout btn btn-info btn-lg">Subscribe</button><button type="button" class="subscribe-activate btn btn-success btn-lg">Activate Subscription</button><button type="button" class="subscribe-cancel btn btn-danger btn-lg">Cancel</button><button type="button" class="subscribe-submit btn btn-success btn-lg">Submit</button>
</div>
