<div class="key-link form-group">
  <label>Temporary url to download key, expires after 24 hours</label>
  <input type="text" class="form-control" readonly>
  <a class="glyphicon glyphicon-download no-select" data-toggle="tooltip" title="Download key"></a>
</div>
<div class="otp-link form-group">
  <label>Temporary url to view key links, expires after 24 hours</label>
  <input type="text" class="form-control" readonly>
  <a class="glyphicon glyphicon-download no-select" data-toggle="tooltip" title="Open key link" target="_blank"></a>
</div>
<div class="uri-link form-group">
  <label>Temporary uri link for Pritunl Client, expires after 24 hours</label>
  <input type="text" class="form-control" readonly>
  <a class="glyphicon glyphicon-download no-select" data-toggle="tooltip" title="Open key uri link" target="_blank"></a>
</div>
