<div class="form-group name">
  <label>Name</label>
  <input type="text" class="form-control" placeholder="Enter name" data-toggle="tooltip" title="Name of user" value="<%- name %>" autofocus>
</div>
<div class="form-group email">
  <label>Email (optional)</label>
  <input type="email" class="form-control" placeholder="Enter email address" data-toggle="tooltip" title="User email address" value="<%- email ? email : '' %>">
</div>
