<div class="split">
  <div class="left">
    <div class="node-host form-group" <%= type === 'node_server' ? '' : 'style="display: none;"' %>>
      <label>Node Host</label>
      <input type="text" class="form-control" placeholder="Enter node host" data-toggle="tooltip" title="Node server host" value="<%- node_port ? node_host + ':' + node_port : node_host %>">
    </div>
    <div class="name form-group">
      <label>Name</label>
      <input type="text" class="form-control" placeholder="Enter name" data-toggle="tooltip" title="Name of VPN server" value="<%- name %>" autofocus>
    </div>
    <div class="network form-group">
      <label>Network</label>
      <span class="label label-default no-select" toggle="tooltip" title="Maximum number of users for this subnet"></span>
      <input type="text" class="form-control" placeholder="Enter network address" data-toggle="tooltip" title="Network address for the private network that will be created for VPN clients" value="<%- network %>">
    </div>
    <div class="interface form-group">
      <label>Interface</label>
      <input type="text" class="form-control" placeholder="Enter tunnel interface" data-toggle="tooltip" title="Tunnel interface that will be created for VPN connections" spellcheck="false" value="<%- interface %>">
    </div>
    <div class="port-protocol form-group">
      <label>Port</label>
      <label>Protocol</label>
      <br>
      <input type="text" class="port form-control" placeholder="Enter port number" data-toggle="tooltip" title="Port number for VPN connections" value="<%- port %>">
      <select class="protocol form-control" data-toggle="tooltip" title="Protocol for VPN connections">
        <option value="udp" <%= protocol === 'udp' ? 'selected' : '' %>>udp</option>
        <option value="tcp" <%= protocol === 'tcp' ? 'selected' : '' %>>tcp</option>
      </select>
    </div>
    <div class="dh-param-bits form-group">
      <label>DH Param Bits</label>
      <select class="form-control" data-toggle="tooltip" title="Size of DH parameters">
        <option value="1024" <%= dh_param_bits === 1024 ? 'selected' : '' %>>1024</option>
        <option value="1536" <%= dh_param_bits === 1536 ? 'selected' : '' %>>1536</option>
        <option value="2048" <%= dh_param_bits === 2048 ? 'selected' : '' %>>2048</option>
        <option value="3072" <%= dh_param_bits === 3072 ? 'selected' : '' %>>3072</option>
        <option value="4096" <%= dh_param_bits === 4096 ? 'selected' : '' %>>4096</option>
      </select>
    </div>
  </div>
  <div class="right">
    <div class="node-key form-group" <%= type === 'node_server' ? '' : 'style="display: none;"' %>>
      <label>Node API Key</label>
      <input type="text" class="form-control" placeholder="Enter api key" data-toggle="tooltip" title="Node server API key" spellcheck="false" value="<%- node_key %>">
    </div>
    <div class="dns-servers form-group">
      <label>DNS Server</label>
      <input type="text" class="form-control" placeholder="Enter DNS server" data-toggle="tooltip" title="Optional, DNS server for clients" value="<%- dns_servers.join(', ') %>">
    </div>
    <div class="search-domain form-group">
      <label>DNS Search Domain</label>
      <input type="text" class="form-control" placeholder="Enter DNS search domain" data-toggle="tooltip" title="Optional, DNS search domain for clients" value="<%- search_domain || '' %>">
    </div>
    <div class="public-address form-group">
      <label>Public IP</label>
      <input type="text" class="form-control" placeholder="Enter public address" data-toggle="tooltip" title="Public IP address or domain name of the VPN server" value="<%- public_address %>">
    </div>
    <div class="server-mode form-group">
      <label>Server Mode</label>
      <select class="form-control" data-toggle="tooltip" title="Forward all of clients internet traffic, only specified local networks or only traffic sent between vpn clients">
        <option value="all_traffic" <%= mode === 'all_traffic' ? 'selected' : '' %>>All Traffic</option>
        <option value="local_traffic" <%= mode === 'local_traffic' ? 'selected' : '' %>>Local Traffic Only</option>
        <option value="vpn_traffic" <%= mode === 'vpn_traffic' ? 'selected' : '' %>>VPN Traffic Only</option>
      </select>
    </div>
    <div class="local-network form-group" <%= mode === 'local_traffic' ? '' : 'style="display: none;"' %>>
      <label>Local Networks</label>
      <input type="text">
    </div>
    <div class="debug-toggle toggle-form form-group">
      <label>Enable Debugging Output</label>
      <div class="selector <%= debug ? 'selected' : '' %> no-select" data-toggle="tooltip" title="Show server debugging information in output">
        <span class="selector-inner glyphicon glyphicon-ok"></span>
      </div>
    </div>
    <div class="otp-auth-toggle toggle-form form-group">
      <label>Enable Two-Step Authentication</label>
      <div class="selector <%= otp_auth ? 'selected' : '' %> no-select" data-toggle="tooltip" title="Enable two-step authentication using Google Authenticator. Verification code is entered as the user password when connecting">
        <span class="selector-inner glyphicon glyphicon-ok"></span>
      </div>
    </div>
  </div>
</div>
