.title {
  font-weight: bold;
  font-family: 'Ubuntu', sans-serif;
}

.title a,
.title a:hover {
  color: @dark-fg;
}

.title a.disabled,
.title a.disabled:hover {
  color: @light-fg;
  pointer-events: none;
  cursor: default;
  text-decoration: none;
}

.container {
  width: 940px;
}

.alert {
  padding: 10px;
  margin-bottom: 10px;
  .transition(background-color, 175ms);
}

.alert.flash {
  background-color: @white-fg;
}

.alert .span {
  vertical-align: text-top;
}

.alert-dismissable .close {
  top: 4px;
  right: 4px;
  opacity: 0.5;
  height: 10px;
  line-height: 10px;
  vertical-align: text-top;
}

.alert-success {
  background-color: @alert-success-bg;
  border-color: @alert-success-border;
}

.alert-info {
  background-color: @alert-info-bg;
  border-color: @alert-info-border;
}

.alert-warning {
  background-color: @alert-warning-bg;
  border-color: @alert-warning-border;
}

.alert-danger {
  background-color: @alert-danger-bg;
  border-color: @alert-danger-border;
}

.header-button {
  margin-top: 2px;
}

.select2-drop {
  margin-top: -3px;
}

.select2-container-multi .select2-choices .select2-search-choice {
  padding: 4px 5px 3px 18px;
}

.select2-container-multi .select2-choices {
  padding: 2px 3px 0 0;
  background-image: none;
  .border-radius(@border-radius);
}

.select2-search-choice-close {
  top: 3px;
}

.modal {
  overflow-y: auto;
}

.modal .modal-body {
  padding: 20px 20px 0 20px;
}

.modal .modal-body .split {
  .clearfix();
}

.modal .modal-body .split .left,
.modal .modal-body .split .right {
  margin-right: 20px;
  width: 238px;
}

.modal .modal-body .split .left {
  float: left;
}

.modal .modal-body .split .right {
  float: right;
}

.modal .modal-body .qrcode {
  width: 192px;
  height: 192px;
  margin: 15px auto 0 auto;
}

.modal .modal-dialog {
  width: 600px;
  margin-top: 75px;
  margin-bottom: 75px;
}

.modal .modal-dialog .modal-footer {
  background-color: @light-dark-bg;
  .border-bottom-left-radius(@border-radius);
  .border-bottom-right-radius(@border-radius);
}

.modal label {
  font-weight: normal;
}

.modal .form-group {
  margin: 0 0 20px 0;
}

.modal .form-control {
  width: 200px;
}

.modal .toggle-form,
.modal .toggle-form label {
  cursor: pointer;
}

.modal .modal-user-list li {
  color: @light-dark-fg;
}

.key-link-modal .modal input {
  width: 480px;
  display: inline-block;
}
.key-link-modal .modal .glyphicon {
  font-size: 21px;
  top: 6px;
  margin-left: 3px;
}
.key-link-modal .modal .glyphicon:hover {
  text-decoration: none;
}

.modal .otp-auth-key {
  text-align: center;
}

.modal .otp-auth-key input {
  width: 192px;
  margin: 0 auto;
}

.modal .generate-new-key {
  margin-top: 15px;
  width: 192px;
}

.modal .generate-new-api-key {
  margin-top: 25px;
  margin-left: 42px;
  width: 200px;
}

.modal .modal-body .postmark-settings {
  display: none;
}
.enterprise .modal .modal-body .postmark-settings {
  display: block;
}
.modal .modal-body .split .settings-right,
.modal .modal-body .split .postmark-right {
  width: 264px;
}
.modal .modal-body .split .settings-left input,
.modal .modal-body .split .postmark-left input {
  width: 250px;
}
.modal .modal-body .split .settings-right input,
.modal .modal-body .split .postmark-right input {
  width: 284px;
  font-family: 'Lucida Console', Monaco, monospace;
}
.settings-modal .postmark-logo {
  margin: 10px 0 10px 211px;
  border: 1px solid @yellow-border;
  width: 135px;
  .border-radius(@border-radius);
}

.navbar .navbar-brand,
.navbar .navbar-brand:hover,
.navbar .navbar-brand:active,
.login .login-header {
  font-family: 'Fredoka One', cursive;
  padding: 13px 15px 0 15px;
  font-size: 28px;
  cursor: default;
  color: @header-fg;
}

.login .login-header.enterprise-header {
  display: none;
}

.enterprise .login .login-header {
  display: none;
}

.enterprise .login .login-header.enterprise-header {
  display: block;
}

.enterprise-clear .login .login-header {
  display: block;
}

.enterprise-clear .login .login-header.enterprise-header {
  display: none;
}

.navbar .navbar-brand.enterprise-brand {
  display: none;
}

.enterprise .navbar .navbar-brand.brand {
  display: none;
}

.enterprise .navbar .navbar-brand.enterprise-brand {
  display: block;
}

.enterprise-clear .navbar .navbar-brand.brand {
  display: block;
}

.enterprise-clear .navbar .navbar-brand.enterprise-brand {
  display: none;
}

.navbar .enterprise-upgrade a {
  padding-top: 13px;
}

.navbar .enterprise-upgrade span {
  background-color: @red-bg;
  padding: 4px;
}

.enterprise .navbar .enterprise-upgrade {
  display: none;
}

.navbar .enterprise-settings {
  display: none;
}

.enterprise .navbar .enterprise-settings {
  display: block;
}

.enterprise-clear .navbar .enterprise-settings {
  display: none;
}

.navbar .nav a {
  cursor: pointer;
}

.selector {
  width: 19px;
  height: 19px;
  margin-right: 10px;
  padding-left: 1px;
  border: 1px solid @light-dark-border;
  float: left;
  .border-radius(@border-radius);
}

.selector .selector-inner {
  font-size: 14px;
  top: auto;
  color: @checkmark-fg;
  display: none;
}

.selector.selected .selector-inner {
  display: inline;
}

.selector:active {
  .box-shadow2(0 1px 2px rgba(0, 0, 0, 0.05),
    0 1px 3px rgba(0, 0, 0, 0.1) inset);
  .background-clip-padding;
}




header.blur {
  .blur(5px);
}

#app.blur {
  .blur(10px);
}

.navbar {
  height: 50px;
  overflow-x: hidden;
  overflow-y: hidden;
  .border-radius(0);
}

.navbar-collapse {
  padding-right: 0;
}




.login,
.login-backdrop {
  position: fixed;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  z-index: 2050;
}

.login-backdrop {
  background-color: @black-bg;
  opacity: 0.75;
  z-index: 2000;
  cursor: default;
}

.login .login-box {
  position: fixed;
  width: 374px;
  margin: 0 auto;
  opacity: 1;
  border: 1px solid rgba(0, 0, 0, 0.2);
  .border-radius(@border-radius);
  .box-shadow(0 5px 15px rgba(0, 0, 0, 0.5));
  .top-calc(~'50% - 164px');
  .left-calc(~'50% - 187px');
}

.login .login-form {
  background-color: @light-dark2-bg;
  padding: 40px 35px;
  .border-bottom-left-radius(@border-radius);
  .border-bottom-right-radius(@border-radius);
}

.login .login-header {
  margin: 0;
  padding: 10px 15px 10px 15px;
  .border-top-left-radius(@border-radius);
  .border-top-right-radius(@border-radius);
}

.login .form-control {
  position: relative;
  font-size: 16px;
  height: auto;
  padding: 10px;
}

.login .form-control:focus {
  z-index: 2;
}

.login .form-control[type="text"] {
  margin-bottom: -1px;
  .border-bottom-left-radius(0);
  .border-bottom-right-radius(0);
}

.login .form-control[type="password"] {
  margin-bottom: 10px;
  .border-top-left-radius(0);
  .border-top-right-radius(0);
}

.login .login-button {
  margin-top: 13px;
}



.orgs-list .orgs-list-title,
.servers-list .servers-list-title {
  color: @dark-fg;
  display: inline-block;
  margin-top: 6px;
  margin-bottom: 19px;
  font-size: 22px;
}

.orgs-list .orgs-list-buttons,
.servers-list .servers-list-buttons {
  float: right;
  display: block;
}

.orgs-list .orgs-add-user-bulk {
  display: none;
}

.enterprise .orgs-list .orgs-add-user-bulk {
  display: inline-block;
}

.orgs-list .orgs-email-selected {
  display: none;
}

.enterprise .orgs-list .orgs-email-selected {
  display: inline-block;
}



.users-list,
.servers-list .server,
.orgs-list-container .no-orgs,
.servers-list .no-servers {
  margin-bottom: 25px;
  border: 1px solid @light-dark-border;
  background-color: @light-dark2-bg;
  .border-radius(@border-radius);
  .box-shadow2(0 2px 1px rgba(0, 0, 0, 0.05),
    0 -1px 2px rgba(0, 0, 0, 0.05) inset);
  .background-clip-padding;
}

.users-list .label,
.servers-list .server .label {
  cursor: default;
  margin-top: 1px;
  margin-right: 14px;
  padding: 6px 12px;
}

.users-list .label.user-count {
  float: right;
  margin-top: 6px;
  margin-right: 12px;
  padding: 4px 8px;
}

.users-list .org-search {
  float: right;
  width: 150px;
  height: 30px;
  margin-right: 2px;
  display: none;
}

.enterprise .users-list .org-search {
  display: block;
}



.users-list .users-list-title,
.servers-list .server .server-title {
  margin: 0;
  display: inline-block;
  line-height: 16px;
  position: relative;
  top: 2px;
}

.users-list .users-list-title a,
.servers-list .server .server-title a {
  font-size: 19px;
}

.users-list .users-list-header,
.servers-list .server .server-header {
  background-color: @white-fg;
  padding: 10px 11px 16px 14px;
  border-bottom: 1px solid @light-dark2-border;
  .border-top-left-radius(@border-radius);
  .border-top-right-radius(@border-radius);
}

.users-list .users-list-header button,
.servers-list .server .server-header button {
  margin-left: 10px;
  float: right;
}

.users-list {
  overflow: hidden;
}

.users-list .prev-page,
.users-list .next-page {
  margin: 12px 0;
  min-width: 105px;
}

.users-list .prev-page {
  float: left;
}

.users-list .next-page {
  float: right;
}

.users-list .search-time {
  margin-top: 10px;
  margin-right: 2px;
  text-align: right;
  color: @light2-fg;
}

.users-list .search-more {
  cursor: pointer;
  margin-top: -20px;
  text-align: center;
}

.users-list .prev-page,
.users-list .next-page,
.users-list .pages,
.users-list .search-time,
.users-list .search-more {
  display: none;
}

.users-list .pages {
  margin-top: 16px;
  text-align: center;
  color: @light2-fg;
}

.users-list .pages.padded-left {
  margin-left: 105px;
}

.users-list .pages.padded-right {
  margin-right: 105px;
}

.users-list .pages .links {
  display: inline-block;
}

.users-list .pages .link,
.users-list .pages .link:hover {
  padding: 0 4px;
  cursor: pointer;
  color: @light2-fg;
}

.users-list .pages .link.current,
.users-list .pages .link.current:hover {
  cursor: default;
  color: @dark2-fg;
  font-weight: bold;
  text-decoration: none;
}

.users-list-container,
.server-orgs-list {
  padding: 0 13px 13px 13px;
}

.orgs-list-container .no-orgs,
.servers-list .no-servers {
  text-align: center;
  font-size: 14px;
  padding: 10px;
  color: @light-dark-fg;
  background-color: @light-dark-bg;
  display: none;
}

.servers-list .server .no-org-warning {
  float: right;
  padding: 10px 12px 10px 12px;
  margin-top: 0;
  margin-right: 0;
  height: 30px;
  display: none;
}




.users-list .user,
.users-list .no-users,
.users-list .no-users-search,
.users-list .loading,
.server-orgs-list .org,
.server-orgs-list .no-orgs {
  height: 40px;
  background: @white-fg;
  border: 1px solid @light-dark-border;
  margin-top: 12px;
  .border-radius(@border-radius);
  .box-shadow2(0 1px 0 rgba(255, 255, 255, 0.02)
    0 1px 3px rgba(0, 0, 0, 0.08) inset);
  .background-clip-padding;
}

.users-list .no-users,
.users-list .no-users-search,
.users-list .loading,
.server-orgs-list .no-orgs {
  text-align: center;
  font-size: 14px;
  color: @light-dark-fg;
  background-color: @light-dark-bg;
  padding-top: 10px;
  display: none;
}

.users-list .loading {
  display: none;
}

.users-list .org-title,
.servers-list .server .server-title a {
  cursor: pointer;
}

.users-list .user .selector {
  margin: 10px 10px;
}




.users-list .user .name-container,
.server-orgs-list .org .name-container {
  float: left;
  height: auto;
  padding-top: 0;
  padding-left: 13px;
  margin-top: 3px;
  border-left: 1px solid @light-dark-border;
}

.users-list .user .name-container {
  margin-top: 5px;
}

.users-list .user-server .info-container {
  margin-top: 5px;
  float: left;
  cursor: default;
}

.users-list .user-server .info-container .title {
  display: inline-block;
}

.users-list .user-server .info-container .server-name .title {
  min-width: 130px;
  padding-right: 2px;
}

.users-list .user-server .info-container .server-addr .title {
  width: 115px;
}


.users-list .user-server .info-container .server-sent .title,
.users-list .user-server .info-container .server-recv .title {
  width: 80px;
}

.users-list .user-server .info-container .server-time .title {
  width: 110px;
}

.server-orgs-list .org .name-container {
  border-left: 0 none;
}

.users-list .user .name-container .name-icon,
.server-orgs-list .org .name-container .name-icon {
  top: 3px;
  margin-top: 0;
  margin-right: 6px;
  font-size: 19px;
  color: @icon-blue-bg;
  vertical-align: text-top;
}

.users-list .user .name-container.type-server .name-icon,
.users-list .user .name-container.type-server .user-name {
  color: @icon-gray-bg;
}

.users-list .user .name-container.type-server .user-name {
  top: 5px;
  font-size: 10px;
}

.users-list .user .name-container .name-icon {
  top: 2px;
}

.users-list .user .name-container .user-name {
  position: relative;
  top: -2px;
}

.users-list .user .name-container .user-name,
.server-orgs-list .org .org-name {
  font-size: 18px;
  cursor: pointer;
  vertical-align: text-top;
}

.users-list .users-list-title .org-id,
.users-list .user .name-container .user-id,
.servers-list .server .server-title .server-id,
.servers-list .server .org .org-id {
  position: relative;
  color: @dark-fg;
  font-size: 10px;
  display: none;
}

.servers-list .server .org .org-id {
  top: 4px;
}

.users-list .user .name-container .user-id {
  top: 2px;
}

.users-list .users-list-title .org-id,
.servers-list .server .server-title .server-id {
  top: -2px;
}

.users-list.show-hidden .users-list-title .org-id,
.users-list.show-hidden .user .name-container .user-id,
.servers-list .server.show-hidden .server-title .server-id,
.servers-list .server.show-hidden .org .org-id {
  display: inline;
}

.server-orgs-list .org .org-name {
  color: @blue-fg;
  cursor: default;
}

.users-list .right-container a {
  font-size: 20px;
  color: @icon-blue2-bg;
  cursor: pointer;
  margin-left: 3px;
}

.users-list .right-container a.disabled {
  pointer-events: none;
  cursor: not-allowed;
  opacity: .65;
}

.users-list .right-container a:hover {
  text-decoration: none;
}

.users-list .user .toggle-servers {
  display: none;
}

.enterprise .users-list .user .toggle-servers {
  display: inline-block;
}

.users-list .user .get-otp-auth.no-otp-auth {
  display: none;
}

.users-list .user .right-container .download-key:hover,
.users-list .user .right-container .get-key-link:hover,
.users-list .user .right-container .get-otp-auth:hover,
.users-list .user .right-container .get-uri-auth:hover,
.users-list .user .right-container .toggle-servers:hover {
  text-decoration: none;
  color: @icon-blue3-bg;
}

.users-list .right-container,
.server-orgs-list .org .button-container {
  float: right;
  display: inline-block;
  margin-top: 5px;
  height: 28px;
  padding: 3px 8px 0 12px;
  white-space: nowrap;
  border-left: 1px solid @light-dark-border;
}

.users-list .user-server .right-container {
  height: auto;
  margin-top: 5px;
  border-left: none;
  padding: 0;
  width: 77px;
}

.server-orgs-list .org .button-container {
  border-left: 0 none;
}

.users-list .user .status-container {
  display: inline-block;
  cursor: default;
  position: relative;
  top: -4px;
  width: 84px;
}

.users-list .status-container .status-icon {
  display: inline-block;
  position: relative;
  top: 3px;
  margin-right: 4px;
  width: 16px;
  height: 16px;
  .border-radius(50%);
}

.users-list .user-server .status-container .status-icon {
  top: 1px;
  width: 14px;
  height: 14px;
}

.users-list .user .tooltip-inner {
    white-space: pre-wrap;
}


.users-list .status-container .status-icon.online {
  background-color: @green-fg;
}

.users-list .status-container .status-icon.offline {
  background-color: @red-fg;
}

.users-list .status-container .status-icon.disabled {
  background-color: @gray-bg;
}

.users-list .status-container .status-text {
  color: @gray-fg;
}

.users-list .user-servers {
  border: 1px solid @light-dark-border;
  border-top: none;
  background: #FFF;
  margin-left: 3px;
  margin-right: 3px;
  padding: 3px 10px;
  display: none;
  .border-bottom-left-radius(@border-radius);
  .border-bottom-right-radius(@border-radius);
  .box-shadow(inset 0px 12px 8px -14px rgba(0, 0, 0, 0.53));
}

.users-list .user-servers .user-server {
  color: @dark-fg;
  font-size: 15px;
  height: 32px;
  border-top: 1px solid @light-dark-border;
}

.users-list .user-servers .no-servers {
  color: @dark-fg;
  font-size: 15px;
  padding: 4px 0;
  display: none;
}

.users-list .user-servers .user-server:first-child {
  border-top: none;
}

.users-list .user-servers .user-server .glyphicon {
  top: 3px;
  margin-right: 0;
}



.dashboard .status-container,
.dashboard .log-container {
  margin-bottom: 25px;
  border: 1px solid @light-dark-border;
  background-color: @light-dark2-bg;
  .border-radius(@border-radius);
  .box-shadow2(0 2px 1px rgba(0, 0, 0, 0.05),
    0 -1px 2px rgba(0, 0, 0, 0.05) inset);
  .background-clip-padding;
}

.dashboard .status-container {
  margin-bottom: 15px;
}

.servers-list .server .server-info {
  background-color: @white-fg;
}

.dashboard .status-container table {
  width: 100%;
  table-layout: fixed;
}

.servers-list .server .server-info .status-items {
  margin-right: 11px;
  width: 300px;
  float: left;
}

.servers-list .server .server-info .server-output {
  border-bottom: 1px solid @light-dark-border;
}

.servers-list .server .server-info .server-output-title {
  color: @dark-fg;
  font-size: 16px;
  margin: 0;
  padding-top: 4px;
  padding-right: 10px;
  height: 24px;
}

.servers-list .server .server-info .server-output-title .btn {
  padding: 0 5px;
}

.servers-list .server .server-info .server-graph-btn {
  display: none;
}

.enterprise .servers-list .server .server-info .server-graph-btn {
  display: inline-block;
}

.servers-list .server .server-info .server-output-clear {
  float: right;
  color: @dark-fg;
  top: 4px;
  font-size: 12px;
  margin-right: 2px;
}

.servers-list .server .server-info .server-graph-period {
  margin-left: 4px;
  float: right;
  display: none;
}

.servers-list .server .server-info .server-output-clear:hover {
  color: @black-bg;
}

.servers-list .server .server-info .server-output-viewer {
  padding: 5px 10px 10px 0;
}

.servers-list .server .server-info .server-graph-viewer {
  margin: 5px 10px 10px 311px;
  overflow: hidden;
  display: none;
}

.servers-list .server .server-info .server-graph-recv,
.servers-list .server .server-info .server-graph-sent {
  width: 587px;
  height: 130px;
  border: 1px solid @light-dark-border;
}

.servers-list .node-server .server-info .server-graph-recv,
.servers-list .node-server .server-info  .server-graph-sent {
  width: 587px;
  height: 151px;
  border: 1px solid @light-dark-border;
}

.servers-list .node-server .server-info  .server-graph-sent {
  height: 152px;
}

.servers-list .server .server-info .server-graph-viewer .server-graph-sent {
  margin-top: 2px;
}

.servers-list .server .server-info .server-graph-viewer .detail .dot,
.servers-list .server .server-info .server-graph-viewer .detail .item {
  display: none;
}

.servers-list .server .server-info .server-graph-viewer .detail .x_label {
  opacity: 0.8;
}

.servers-list .server .server-info .text-viewer {
  height: 262px;
  border: 1px solid @light-dark-border;
}

.servers-list .node-server .server-info .text-viewer {
  height: 305px;
}

.dashboard .status-container .status-item,
.servers-list .server .server-info .status-item {
  padding: 20px 0;
  border-right: 1px solid @light-dark-border;
  text-align: center;
}

.servers-list .server .server-info .status-item {
  height: 43px;
  padding: 9px 12px;
  text-align: left;
  border-bottom: 1px solid @light-dark-border;
}

.servers-list .server .server-info .status-item.last {
 border-bottom: 0 none;
}

.dashboard .status-container .status-item.last {
  border-right: 0 none;
}

.dashboard .status-container .status-item span {
  vertical-align: top;
}

.dashboard .status-container .status-item .status-title,
.servers-list .server .server-info .status-item .status-title {
  color: @dark-fg;
  font-size: 26px;
  margin: 0 0 3px 0;
}

.servers-list .server .server-info .status-item .status-title {
  font-size: 20px;
  height: 22px;
  margin: 0;
}

.servers-list .server .server-info .status-item .status-title span {
  top: 2px;
  vertical-align: text-top;
}

.dashboard .status-container .status-item .status-icon,
.servers-list .server .server-info .status-item .status-icon {
  color: @icon-blue4-bg;
  font-size: 26px;
}

.servers-list .server .server-info .status-item .status-icon {
  font-size: 18px;
  margin-right: 0;
}

.dashboard .status-container .status-item .status-icon.glyphicon-cloud {
  top: 4px;
}

.servers-list .server .server-info .server-port .status-icon {
  top: 0;
}

.servers-list .server .server-info .status-item .status-text {
  color: @dark-fg;
  float: right;
  position: relative;
  top: -21px;
}

.dashboard .status-container .status-text {
  font-size: 16px;
  font-weight: normal;
}

.dashboard .status-container .status-text .status-num {
  font-weight: bold;
  position: relative;
  top: 1px;
}

.dashboard .status-container .server-version {
  position: absolute;
  top: -1px;
  right: 2px;
  font-size: 12px;
  z-index: 5000;
}

.dashboard .log-container {
  padding: 20px 20px 5px 20px;
}

.dashboard .log-container .log-title {
  color: @dark-fg;
  margin: 0;
  font-size: 20px;
  padding-bottom: 17px;
  border-bottom: 1px solid @light-dark-border;
}

.dashboard .log-container .log-entry {
  color: @dark-fg;
  padding: 5px 15px 5px 10px;
  border-bottom: 1px solid @light-dark-border;
}

.dashboard .log-container .log-entry.last {
  border-bottom: 0 none;
}

.dashboard .log-container .log-entry span {
  vertical-align: top;
}

.dashboard .log-container .log-entry .log-icon {
  color: @icon-blue4-bg;
  font-size: 15px;
  margin-right: 7px;
  float: left;
  top: 3px;
}

.dashboard .log-container .log-entry .log-msg {
  float: left;
}

.dashboard .log-container .log-entry .log-time {
  color: @light-dark-fg;
  float: right;
}

.notification-modal .notification {
  font-size: 16px;
  font-weight: bold;
  text-align: center;
}



.server-settings-modal .modal .port-protocol label,
.server-settings-modal .modal .port-protocol input,
.server-settings-modal .modal .port-protocol select {
  width: 98px;
}

.server-settings-modal .modal .port-protocol input,
.server-settings-modal .modal .port-protocol select {
  display: inline-block;
}

.server-settings-modal .modal .form-group {
  margin: 0 0 15px 0;
}

.server-settings-modal .modal .network {
  position: relative;
}

.server-settings-modal .modal .network .label {
  cursor: default;
  position: absolute;
  top: 34px;
  right: 47px;
  padding: 2px 3px;
}

.server-settings-modal .modal .left .otp-auth-toggle {
  margin-top: 17px;
}

.server-settings-modal .modal .right .otp-auth-toggle label {
  margin-bottom: 0;
}

.subscribe-modal .modal .subscribe-info,
.enterprise-modal .modal .enterprise-info {
  margin: 0 auto;
  width: 353px;
  text-align: center;
}

.enterprise-modal .modal .enterprise-info {
  width: 280px;
  text-align: left;
}

.subscribe-modal .modal .subscribe-activate-form {
  display: none;
  text-align: center;
}

.enterprise-modal .modal .enterprise-info .enterprise-item {
  float: right;
  text-align: right;
}

.subscribe-modal .modal .subscribe-activate-form label,
.enterprise-modal .modal .enterprise-list span {
  text-align: center;
  color: @gray2-fg;
  font-weight: bold;
  font-family: 'Ubuntu', sans-serif;
  font-size: 19px;
}

.enterprise-modal .modal .enterprise-list span {
  font-size: 22px;
}

.subscribe-modal .modal .subscribe-activate-form textarea {
  resize: none;
  font-family: 'Lucida Console', Monaco, monospace;
  .border-radius(@border-radius);
  .box-shadow(0px 0px 14px @gray4-fg);
}

.add-user-bulk-modal .modal textarea {
  width: 100%;
  resize: none;
  font-family: 'Lucida Console', Monaco, monospace;
}

.subscribe-modal .modal .subscribe-buttons,
.enterprise-modal .modal .enterprise-buttons {
  margin: 5px 5px 10px 5px;
  text-align: center;
}

.subscribe-modal .modal .subscribe-buttons .subscribe-cancel,
.subscribe-modal .modal .subscribe-buttons .subscribe-submit {
  display: none;
}

.subscribe-modal .modal .subscribe-buttons button,
.enterprise-modal .modal .enterprise-buttons button {
  margin: 5px;
  width: 225px;
}

.enterprise-modal .modal .enterprise-buttons .enterprise-reactivate {
  display: none;
}

.subscribe-modal .modal .subscribe-list {
  border: 1px solid @gray3-fg;
  list-style-type: none;
  padding: 0;
  text-align: left;
  margin-top: 5px;
  margin-bottom: 15px;
  .border-radius(@border-radius);
  .box-shadow(0px 0px 14px @gray4-fg);
}

.enterprise-modal .modal .enterprise-list {
  padding-left: 0;
  list-style-type: none;
}

.subscribe-modal .modal .subscribe-list li {
  border-top: 1px solid @light-fg;
  padding: 16px 20px;
}

.subscribe-modal .modal .subscribe-list li:first-child {
  border-top: none;
}

.subscribe-modal .modal .subscribe-list li .glyphicon {
  margin-right: 15px;
  font-size: 18px;
  color: @blue-fg;
}

.subscribe-modal .modal .subscribe-list li .subscribe-feature {
  color: @gray2-fg;
  font-weight: bold;
  font-family: 'Ubuntu', sans-serif;
  font-size: 18px;
}
