<div class="enterprise-info form-group">
  <ul class="enterprise-list">
    <li class="status">
      <span class="enterprise-label">Status:</span>
      <span class="enterprise-item"></span>
    </li>
    <li class="plan">
      <span class="enterprise-label">Plan:</span>
      <span class="enterprise-item">Enterprise</span>
    </li>
    <li class="amount">
      <span class="enterprise-label">Amount:</span>
      <span class="enterprise-item"></span>
    </li>
    <li class="renew">
      <span class="enterprise-label">Renew:</span>
      <span class="enterprise-item"></span>
    </li>
  </ul>
</div>
<div class="enterprise-buttons">
  <button type="button" class="enterprise-update btn btn-primary btn-lg">Update Payment Info</button>
  <button type="button" id="trigger-uservoice" class="enterprise-support btn btn-info btn-lg">Contact Support</button><br>
  <button type="button" class="enterprise-cancel btn btn-danger btn-lg first">Cancel Subscription</button>
  <button type="button" class="enterprise-reactivate btn btn-success btn-lg first">Reactivate Subscription</button>
  <button type="button" class="enterprise-remove btn btn-warning btn-lg">Remove License</button><br>
</div>
