<div class="info-container">
  <span class="server-item server-name" data-toggle="tooltip" title="Server name">
    <span class="name-icon glyphicon glyphicon-dashboard"></span>
    <span class="title"></span>
  </span>
  <span class="server-item server-addr" data-toggle="tooltip" title="User IP address">
    <span class="name-icon glyphicon glyphicon-record"></span>
    <span class="title"></span>
  </span>
  <span class="server-item server-sent" data-toggle="tooltip" title="Data sent">
    <span class="name-icon glyphicon glyphicon-upload"></span>
    <span class="title"></span>
  </span>
  <span class="server-item server-recv" data-toggle="tooltip" title="Data received">
    <span class="name-icon glyphicon glyphicon-download"></span>
    <span class="title"></span>
  </span>
  <span class="server-item server-time" data-toggle="tooltip" title="Connected since">
    <span class="name-icon glyphicon glyphicon-time"></span>
    <span class="title"></span>
  </span>
</div>
<div class="right-container">
  <div class="status-container">
    <div class="status-icon"></div>
    <span class="status-text title"></span>
  </div>
</div>
