<div class="no-users">
  There are no users in this organization
</div>
<div class="no-users-search">
  No users found
</div>
<div class="loading">
  Loading...
</div>
<button type="button" class="prev-page btn btn-primary btn-sm">Previous Page</button>
<button type="button" class="next-page btn btn-primary btn-sm">Next Page</button>
<div class="pages no-select">
  <div class="links">
    <a class="link first">First</a>
    <a class="link last">Last</a>
  </div>
</div>
<div class="search-time no-select"></div>
<div class="search-more">
  <a class="no-select">show more results</a>
</div>
