*, *:before, *:after {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}

body {
  background-color: #efefef;
  overflow-y: scroll;
}

a,
a:focus,
a:hover,
a:active {
  outline: 0;
}

select:-moz-focusring {
  color: transparent;
  text-shadow: 0 0 0 #555;
}

ul {
  padding-left: 30px;
}

.form-control[disabled],
.form-control[readonly],
fieldset[disabled] .form-control {
  cursor: default;
}

.alert,
.label,
.btn,
.form-control,
.modal-content,
.select2-drop,
.select2-container-multi .select2-choices .select2-search-choice {
  .border-radius(@border-radius);
}

.label-purple {
  background-color: #a722c0;
}

.default-text {
  color: @gray2-fg !important;
}

.error-text {
  color: @red2-fg !important;
}

.warning-text {
  color: @orange-fg !important;
}

.success-text {
  color: @green2-fg !important;
}

.primary-text-light {
  color: @blue-fg !important;
}

.error-text-light {
  color: @red3-fg !important;
}

.warning-text-light {
  color: @orange2-fg !important;
}

.success-text-light {
  color: @green3-fg !important;
}

@font-face {
  font-family: 'Fredoka One';
  font-style: normal;
  font-weight: 400;
  src: url('../fonts/fredoka-one.eot') format('embedded-opentype'),
    url('../fonts/fredoka-one.woff') format('woff');
}

@font-face {
  font-family: 'Ubuntu';
  font-style: normal;
  font-weight: 400;
  src: url('../fonts/ubuntu.eot') format('embedded-opentype'),
    url('../fonts/ubuntu.woff') format('woff');
}

@font-face {
  font-family: 'Ubuntu';
  font-style: normal;
  font-weight: 700;
  src: url('../fonts/ubuntu-bold.eot') format('embedded-opentype'),
    url('../fonts/ubuntu-bold.woff') format('woff');
}
